/* Market is a collection of assets.
 * For example, Nikkei, Nasdaq, etc.
 * 
 * A market is characterized by all assets being inside the same directory.
 */

package data;

import java.util.*;
import engine.*;
import java.io.*;
import java.text.*;

public class Market {

	private static Market instance;
	
	public String name;
	public Vector<Asset> assets;
	public Vector<Price> index;
	
	protected Market() 
	{
		name = null;
		assets = null;
		index = null;
	}
	
	public static synchronized Market getInstance() 
	{
		if(instance == null) {
			instance = new Market();
	      }	
		return instance;
	}	
	
	
	// Set up a name for the market (for reloading data)
	// not sure if this will be useful yet.
	public void setName(String n)
	{
		name = n;
	}
		
	// reads asset data from a directory
	public void loadDir(String dirname)
	{
		assets = new Vector<Asset>();
		
		File dir = new File (dirname);
		
		// DEBUG: use this to see where the data is being loaded from.
		// System.err.println(dir.getAbsolutePath());
		
		File[] files = dir.listFiles(new ExtensionFilter(".csv"));
		
		for (int i = 0; i < files.length; i++)
			assets.add(new Asset(files[i]));
		
		loadIndex(dir.listFiles(new ExtensionFilter(".idx")));
		
		Collections.sort(assets);
		
		
	}
	
	public void loadIndex(File f[])
	{
		index = new Vector<Price>();
		
		BufferedReader reader;	
       String line = null;	

       try
       {
    	   reader =
    		   new BufferedReader(new FileReader(f[0]));

    	   // skipping first line
    	   line = reader.readLine();
    	   
    	   while ((line = reader.readLine()) != null)
       		{
    		   String[] input = line.split(",");
    	   
    		   SimpleDateFormat dateparser = new SimpleDateFormat("yyyy-MM-dd", new Locale("en"));
    		   Date d = dateparser.parse(input[0]);
    	   
    		   index.add(new Price(d, 
    				   Double.valueOf(input[6]),
    				   Double.valueOf(input[5])));
       		}
    	   
    	   Collections.sort(index);
    	   
    	   
       } catch (Exception e)
       {
    	   System.err.print("Error opening file: " + f[0].getAbsolutePath() + "\n");
    	   System.err.print("Message: " + e.getMessage() + "\n\n");
     	   System.exit(1);
       }
       	       	
	}
	
	// remove all asset data
	public void makeEmpty()
	{
		if (assets != null)
			assets.clear();
	}

	
	public Asset getAssetFromName(String n)
	{
		for (int i = 0; i < assets.size(); i++)
			if (assets.get(i).name.equals(n))
				return assets.get(i);
		return null;
	}

	public int getIndexPosByDate(Date d)
	{			
		for (int i = 0; i < index.size(); i++)
		{
			if (Utils.calcMonths(d, index.get(i).d) == 0)
				return i;
		}
		return -1;	
		
	}
}	


